/**
 * Sets a vector components.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {number} x
 * @param {number} y
 */
export function set2(a: any, i: number, x: number, y: number): void;
/**
 * Sets a vector to another vector.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 */
export function set(a: any, i: number, b: any, j: number): void;
/**
 * Compares two vectors.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @returns {boolean}
 */
export function equals(a: any, i: number, b: any, j: number): boolean;
/**
 * Adds a vector to another.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 */
export function add(a: any, i: number, b: any, j: number): void;
/**
 * Subtracts a vector from another.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 */
export function sub(a: any, i: number, b: any, j: number): void;
/**
 * Scales a vector by a number.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {number} s
 */
export function scale(a: any, i: number, s: number): void;
/**
 * Adds two vectors after scaling the second one.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @param {number} s
 */
export function addScaled(a: any, i: number, b: any, j: number, s: number): void;
/**
 * Calculates the dot product of two vectors.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @returns {number}
 */
export function dot(a: any, i: number, b: any, j: number): number;
/**
 * Calculates the length of a vector.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @returns {number}
 */
export function length(a: any, i: number): number;
/**
 * Calculates the squared length of a vector.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @returns {number}
 */
export function lengthSq(a: any, i: number): number;
/**
 * Normalises a vector.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 */
export function normalize(a: any, i: number): void;
/**
 * Calculates the distance between two vectors.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @returns {number}
 */
export function distance(a: any, i: number, b: any, j: number): number;
/**
 * Calculates the squared distance between two vectors.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @returns {number}
 */
export function distanceSq(a: any, i: number, b: any, j: number): number;
/**
 * Limits a vector to a length.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {number} len
 */
export function limit(a: any, i: number, len: number): void;
/**
 * Linearly interpolates between two vectors.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {import("./types.js").avec2} b
 * @param {number} j
 * @param {number} t
 */
export function lerp(a: any, i: number, b: any, j: number, t: number): void;
/**
 * Executes a function once for each array element.
 * @param {import("./types.js").avec2} a
 * @param {import("./types.js").iterativeCallback} callbackFn
 */
export function forEach(a: any, callbackFn: import("./types.js").iterativeCallback): void;
/**
 * Creates a new array populated with the results of calling a provided function on every element in the calling array.
 * @param {import("./types.js").avec2} a
 * @param {import("./types.js").iterativeCallback} callbackFn
 * @returns {import("./types.js").avec2}
 */
export function map(a: any, callbackFn: import("./types.js").iterativeCallback): any;
/**
 * Prints a vector to a string.
 * @param {import("./types.js").avec2} a
 * @param {number} i
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: any, i: number, precision?: number): string;
