/**
 * Sets a vector components.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {number} x
 * @param {number} y
 * @param {number} z
 * @param {number} w
 */
export function set4(a: import("./types.js").avec4, i: number, x: number, y: number, z: number, w: number): void;
/**
 * Sets a vector to another vector.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 */
export function set(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number): void;
/**
 * Compares two vectors.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 * @returns {boolean}
 */
export function equals(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number): boolean;
/**
 * Adds a vector to another.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 */
export function add(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number): void;
/**
 * Subtracts a vector from another.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 */
export function sub(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number): void;
/**
 * Scales a vector by a number.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {number} s
 */
export function scale(a: import("./types.js").avec4, i: number, s: number): void;
/**
 * Adds two vectors after scaling the second one.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 * @param {number} s
 */
export function addScaled(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number, s: number): void;
/**
 * Multiplies a vector with a matrix.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").amat4} m
 * @param {number} j
 */
export function multMat4(a: import("./types.js").avec4, i: number, m: import("./types.js").amat4, j: number): void;
/**
 * Linearly interpolates between two vectors.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {import("./types.js").avec4} b
 * @param {number} j
 * @param {number} t
 */
export function lerp(a: import("./types.js").avec4, i: number, b: import("./types.js").avec4, j: number, t: number): void;
/**
 * Executes a function once for each array element.
 * @param {import("./types.js").avec4} a
 * @param {import("./types.js").iterativeCallback} callbackFn
 */
export function forEach(a: import("./types.js").avec4, callbackFn: import("./types.js").iterativeCallback): void;
/**
 * Creates a new array populated with the results of calling a provided function on every element in the calling array.
 * @param {import("./types.js").avec4} a
 * @param {import("./types.js").iterativeCallback} callbackFn
 * @returns {import("./types.js").avec4}
 */
export function map(a: import("./types.js").avec4, callbackFn: import("./types.js").iterativeCallback): import("./types.js").avec4;
/**
 * Prints a vector to a string.
 * @param {import("./types.js").avec4} a
 * @param {number} i
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: import("./types.js").avec4, i: number, precision?: number): string;
