/**
 * Returns a new quat at 0, 0, 0, 1.
 * @returns {import("./types.js").quat}
 */
export function create(): import("./types.js").quat;
/**
 * Sets a quaternion to the identity quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function identity(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Returns a copy of a quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function copy(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Multiplies one quaternion by another.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").quat} b
 * @returns {import("./types.js").quat}
 */
export function mult(a: import("./types.js").quat, b: import("./types.js").quat): import("./types.js").quat;
/**
 * Inverts a quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function invert(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Conjugates a quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function conjugate(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Calculates the length of a quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function length(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Normalizes a quaternion.
 * @param {import("./types.js").quat} a
 * @returns {import("./types.js").quat}
 */
export function normalize(a: import("./types.js").quat): import("./types.js").quat;
/**
 * Calculates the dot product of two quaternions.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").quat} b
 * @returns {import("./types.js").quat}
 */
export function dot(a: import("./types.js").quat, b: import("./types.js").quat): import("./types.js").quat;
/**
 * Set euler angles to a quaternion. Assumes XYZ rotation order.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").euler} e
 * @returns {import("./types.js").quat}
 */
export function fromEuler(a: import("./types.js").quat, e: import("./types.js").euler): import("./types.js").quat;
/**
 * Set the angle at an axis of a quaternion.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").vec3} v
 * @param {import("./types.js").Radians} r
 * @returns {import("./types.js").quat}
 */
export function fromAxisAngle(a: import("./types.js").quat, v: import("./types.js").vec3, r: import("./types.js").Radians): import("./types.js").quat;
/**
 * @private
 */
export function _fromMat39(a: any, m0: any, m1: any, m2: any, m3: any, m4: any, m5: any, m6: any, m7: any, m8: any): any;
/**
 * Sets a quaternion from orthonormal base xyz.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").vec3} x
 * @param {import("./types.js").vec3} y
 * @param {import("./types.js").vec3} z
 * @returns {import("./types.js").quat}
 */
export function fromAxes(a: import("./types.js").quat, x: import("./types.js").vec3, y: import("./types.js").vec3, z: import("./types.js").vec3): import("./types.js").quat;
/**
 * Sets a quaternion to a 3x3 matrix.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").mat3} m
 * @returns {import("./types.js").quat}
 */
export function fromMat3(a: import("./types.js").quat, m: import("./types.js").mat3): import("./types.js").quat;
/**
 * Sets a quaternion to a 4x4 matrix.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").mat4} m
 * @returns {import("./types.js").quat}
 */
export function fromMat4(a: import("./types.js").quat, m: import("./types.js").mat4): import("./types.js").quat;
/**
 * Sets a quaternion from a vector to another.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").vec3} eye
 * @param {import("./types.js").vec3} target
 * @param {import("./types.js").vec3} [up=Y_UP]
 * @returns {import("./types.js").quat}
 */
export function targetTo(a: import("./types.js").quat, eye: import("./types.js").vec3, target: import("./types.js").vec3, up?: import("./types.js").vec3): import("./types.js").quat;
/**
 * Spherical linear interpolates between two quaternions.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").quat} b
 * @param {number} t
 * @returns {import("./types.js").quat}
 */
export function slerp(a: import("./types.js").quat, b: import("./types.js").quat, t: number): import("./types.js").quat;
/**
 * Sets a quaternion to another quaternion.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").quat} b
 * @returns {import("./types.js").quat}
 */
export const set: typeof vec4.set;
/**
 * Compares two quaternions.
 * @param {import("./types.js").quat} a
 * @param {import("./types.js").quat} b
 * @returns {boolean}
 */
export const equals: typeof vec4.equals;
export function fromTo(a: any, v: any, w: any): any;
/**
 * Prints a quaternion to a string.
 * @param {import("./types.js").quat} a
 * @param {number} precision
 * @returns {import("./types.js").quat}
 */
export const toString: typeof vec4.toString;
import * as vec4 from "./vec4.js";
