/**
 * Linear interpolation between two numbers.
 * @param {number} a
 * @param {number} b
 * @param {number} t
 * @returns {number}
 */
export function lerp(a: number, b: number, t: number): number;
/**
 * Clamps a number between two numbers.
 * @param {number} n
 * @param {number} min
 * @param {number} max
 * @returns {number}
 */
export function clamp(n: number, min: number, max: number): number;
/**
 * Smooth Hermite interpolation between 0 and 1
 * @param {number} n
 * @param {number} min
 * @param {number} max
 * @returns {number}
 */
export function smoothstep(n: number, min: number, max: number): number;
/**
 * Maps a number from one range to another.
 * @param {number} n
 * @param {number} inStart
 * @param {number} inEnd
 * @param {number} outStart
 * @param {number} outEnd
 * @returns {number}
 */
export function map(n: number, inStart: number, inEnd: number, outStart: number, outEnd: number): number;
/**
 * Transforms degrees into radians.
 * @param {import("./types.js").Degrees} degrees
 * @returns {import("./types.js").Radians}
 */
export function toRadians(degrees: import("./types.js").Degrees): import("./types.js").Radians;
/**
 * Transforms radians into degrees.
 * @param {import("./types.js").Radians} radians
 * @returns {import("./types.js").Degrees}
 */
export function toDegrees(radians: import("./types.js").Radians): import("./types.js").Degrees;
/**
 * Check if a number is a power of two
 * @param {number} a
 * @returns {boolean}
 */
export function isPowerOfTwo(a: number): boolean;
/**
 * Returns the next highest power of two.
 * @param {number} n
 * @returns {number}
 */
export function nextPowerOfTwo(n: number): number;
/**
 * Returns the previous power of two.
 * @param {number} n
 * @returns {number}
 */
export function prevPowerOfTwo(n: number): number;
/** @module utils */
/**
 * @constant {number}
 */
export const EPSILON: 0.000001;
/**
 * @constant {import("./types.js").vec3}
 */
export const Y_UP: readonly number[];
