/** @module vec4 */
/**
 * Returns a new vec4 at 0, 0, 0, 1.
 * @returns {import("./types.js").vec4}
 */
export function create(): import("./types.js").vec4;
/**
 * Returns a copy of a vector.
 * @param {import("./types.js").vec4} a
 * @returns {import("./types.js").vec4}
 */
export function copy(a: import("./types.js").vec4): import("./types.js").vec4;
/**
 * Sets a vector to another vector.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @returns {import("./types.js").vec4}
 */
export function set(a: import("./types.js").vec4, b: import("./types.js").vec4): import("./types.js").vec4;
/**
 * Compares two vectors.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @returns {boolean}
 */
export function equals(a: import("./types.js").vec4, b: import("./types.js").vec4): boolean;
/**
 * Adds a vector to another.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @returns {import("./types.js").vec4}
 */
export function add(a: import("./types.js").vec4, b: import("./types.js").vec4): import("./types.js").vec4;
/**
 * Subtracts a vector from another.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @returns {import("./types.js").vec4}
 */
export function sub(a: import("./types.js").vec4, b: import("./types.js").vec4): import("./types.js").vec4;
/**
 * Scales a vector by a number.
 * @param {import("./types.js").vec4} a
 * @param {number} s
 * @returns {import("./types.js").vec4}
 */
export function scale(a: import("./types.js").vec4, s: number): import("./types.js").vec4;
/**
 * Adds two vectors after scaling the second one.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @param {number} s
 * @returns {import("./types.js").vec4}
 */
export function addScaled(a: import("./types.js").vec4, b: import("./types.js").vec4, s: number): import("./types.js").vec4;
/**
 * Create a vec4 from vec3.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec3} b
 * @returns {import("./types.js").vec4}
 */
export function fromVec3(a: import("./types.js").vec4, b: import("./types.js").vec3): import("./types.js").vec4;
/**
 * Multiplies a vector with a matrix.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").mat4} m
 * @returns {import("./types.js").vec4}
 */
export function multMat4(a: import("./types.js").vec4, m: import("./types.js").mat4): import("./types.js").vec4;
/**
 * Linearly interpolates between two vectors.
 * @param {import("./types.js").vec4} a
 * @param {import("./types.js").vec4} b
 * @param {number} t
 * @returns {import("./types.js").vec4}
 */
export function lerp(a: import("./types.js").vec4, b: import("./types.js").vec4, t: number): import("./types.js").vec4;
/**
 * Prints a vector to a string.
 * @param {import("./types.js").vec4} a
 * @param {number} [precision=4]
 * @returns {string}
 */
export function toString(a: import("./types.js").vec4, precision?: number): string;
